function [gx, gy] = ndc(si, cm, al, be, r)
% function [mag, xx, yy] = ndc(si, cm, al, be, r)
%	Perform normalised differential convolution on an image.

%	si is the irregularly sampled image.
%	cm is the confindence map.
%	al sets mask param
%	be sets mask param
%	r sets mask radius.	
%
% 	See also NDCMASK, ADAPTIVE_NC

if nargin < 5
	al = 3;
	be = 0.5;
	r  = 8;
end

[m, m_x, m_y, m_xy, m_xx, m_yy] = ndcMask(al, be, r);

cm 	  = double(cm); 

nc    = conv2(cm, m, 'same');
cc    = conv2(si, m, 'same');

c_x   = conv2(si, m_x, 'same');
c_y   = conv2(si, m_y, 'same');

nc_x  = conv2(cm, m_x, 'same');
nc_y  = conv2(cm, m_y, 'same');

d_x	  = nc .* c_x - nc_x .* cc;
d_y	  = nc .* c_y - nc_y .* cc;

n_xx  = nc .* conv2(cm, m_xx, 'same') - nc_x.^2;
n_xy  = nc .* conv2(cm, m_xy, 'same') - nc_x .* nc_y;
n_yx  = n_xy; 
n_yy  = nc .* conv2(cm, m_yy, 'same') - nc_y.^2;

[rows, cols] = size(si);
for i = 1:rows
	for j = 1:cols
		D = [d_x(i, j); d_y(i, j)];
		N = [n_xx(i, j), n_xy(i, j); n_yx(i, j), n_yy(i, j)];
		G = inv(N)*D;
		gx(i, j) = G(1, 1);
		gy(i, j) = G(2, 1);
	end
end


